#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import re
import os
import shutil
import logging
import tempfile
import argparse
import ConfigParser
from datetime import datetime
from argparse import ArgumentDefaultsHelpFormatter
from util.adb_helper import AdbHelper
from util.adb_helper import AdbWrapper


logger = logging.getLogger(__name__)


class BackupRestoreHelper(object):
    def __init__(self, **kwargs):
        self._FILE_PROFILE_INI = 'profiles.ini'
        self._FILE_COMPATIBILITY_INI = 'compatibility.ini'
        self._LOCAL_DIR_SDCARD = 'sdcard'
        self._LOCAL_DIR_WIFI = 'wifi'
        self._LOCAL_FILE_WIFI = 'wifi/wpa_supplicant.conf'
        self._LOCAL_DIR_B2G = 'b2g-mozilla'
        self._LOCAL_DIR_DATA = 'data-local'
        self._LOCAL_DIR_DATA_APPS = 'webapps'
        self._REMOTE_DIR_SDCARD = '/sdcard/'
        self._REMOTE_FILE_WIFI = '/data/misc/wifi/wpa_supplicant.conf'
        self._REMOTE_FILE_WIFI_OWNER = 'system:wifi'
        self._REMOTE_DIR_B2G = '/data/b2g/mozilla'
        self._REMOTE_DIR_DATA = '/data/local'

        self.arg_parser = argparse.ArgumentParser(description='Workaround for backing up and restoring Firefox OS profiles. (BETA)',
                                                  formatter_class=ArgumentDefaultsHelpFormatter)
        self.arg_parser.add_argument('-s', '--serial', action='store', dest='serial', default=None, help='Directs command to the device or emulator with the given serial number. Overrides ANDROID_SERIAL environment variable.')
        br_group = self.arg_parser.add_mutually_exclusive_group(required=True)
        br_group.add_argument('-b', '--backup', action='store_true', dest='backup', default=False, help='Backup user profile.')
        br_group.add_argument('-r', '--restore', action='store_true', dest='restore', default=False, help='Restore user profile.')
        self.arg_parser.add_argument('--sdcard', action='store_true', dest='sdcard', default=False, help='Also backup/restore SD card.')
        self.arg_parser.add_argument('--no-reboot', action='store_true', dest='no_reboot', default=False, help='Do not reboot B2G after backup/restore.')
        self.arg_parser.add_argument('-p', '--profile-dir', action='store', dest='profile_dir', default='mozilla-profile', help='Specify the profile folder.')
        self.arg_parser.add_argument('-v', '--verbose', action='store_true', dest='verbose', default=False, help='Turn on verbose output, with all the debug logger.')
        self.args = self.arg_parser.parse_args()
        # setup the logging config
        if self.args.verbose is True:
            verbose_formatter = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
            logging.basicConfig(level=logging.DEBUG, format=verbose_formatter)
        else:
            formatter = '%(levelname)s: %(message)s'
            logging.basicConfig(level=logging.INFO, format=formatter)
        AdbWrapper.check_adb()

    def stop_b2g(self, serial=None):
        logger.info('Stop B2G.')
        output, retcode = AdbWrapper.adb_shell('stop b2g', serial=serial)

    def start_b2g(self, serial=None):
        logger.info('Start B2G.')
        output, retcode = AdbWrapper.adb_shell('start b2g', serial=serial)

    def backup_sdcard(self, local_dir, serial=None):
        logger.info('Backing up SD card...')
        # try to get the /sdcard folder on device
        output, retcode = AdbWrapper.adb_shell('ls -d {0}; echo $?'.format(self._REMOTE_DIR_SDCARD), serial=serial)
        output_list = [item for item in re.split(r'\n+', re.sub(r'\r+', '', output)) if item]
        ret_code = output_list[-1]
        output_list.remove(output_list[-1])
        ret_msg = '\n'.join(output_list)
        if ret_code == '0':
            target_dir = local_dir + os.sep + self._LOCAL_DIR_SDCARD + os.sep
            os.makedirs(target_dir)
            logger.info('Backup: {0} to {1}'.format(self._REMOTE_DIR_SDCARD, target_dir))
            try:
                AdbWrapper.adb_pull(self._REMOTE_DIR_SDCARD, target_dir, serial=serial)
            except:
                logger.warning('Can not pull files from {0} to {1}'.format(self._REMOTE_DIR_SDCARD, target_dir))
        else:
            logger.info(ret_msg)
        logger.info('Backup SD card done.')

    def restore_sdcard(self, local_dir, serial=None):
        logger.info('Restoring SD card...')
        target_dir = local_dir + os.sep + self._LOCAL_DIR_SDCARD
        if os.path.isdir(target_dir):
            logger.info('Restore: {0} to {1}'.format(target_dir, self._REMOTE_DIR_SDCARD))
            try:
                AdbWrapper.adb_push(target_dir, self._REMOTE_DIR_SDCARD, serial=serial)
            except:
                logger.warning('Can not push files from {0} to {1}'.format(target_dir, self._REMOTE_DIR_SDCARD))
        else:
            logger.info('{0}: No such file or directory'.format(target_dir))
            return
        logger.info('Restore SD card done.')

    def backup_profile(self, local_dir, serial=None):
        logger.info('Backing up profile...')
        # Backup Wifi
        wifi_dir = local_dir + os.sep + self._LOCAL_DIR_WIFI + os.sep
        wifi_file = local_dir + os.sep + self._LOCAL_FILE_WIFI
        os.makedirs(wifi_dir)
        logger.info('Backing up Wifi information...')
        try:
            AdbWrapper.adb_pull(self._REMOTE_FILE_WIFI, wifi_file, serial=serial)
        except:
            logger.warning('If you don\'t have root permission, you cannot backup Wifi information.')
        # Backup profile
        b2g_mozilla_dir = local_dir + os.sep + self._LOCAL_DIR_B2G + os.sep
        os.makedirs(b2g_mozilla_dir)
        logger.info('Backing up {0} to {1} ...'.format(self._REMOTE_DIR_B2G, b2g_mozilla_dir))
        try:
            AdbWrapper.adb_pull(self._REMOTE_DIR_B2G, b2g_mozilla_dir, serial=serial)
        except:
            logger.warning('Can not pull files from {0} to {1}'.format(self._REMOTE_DIR_B2G, b2g_mozilla_dir))
        # Backup data/local
        datalocal_dir = local_dir + os.sep + self._LOCAL_DIR_DATA + os.sep
        os.makedirs(datalocal_dir)
        logger.info('Backing up {0} to {1} ...'.format(self._REMOTE_DIR_DATA, datalocal_dir))
        try:
            AdbWrapper.adb_pull(self._REMOTE_DIR_DATA, datalocal_dir, serial=serial)
        except:
            logger.warning('Can not pull files from {0} to {1}'.format(self._REMOTE_DIR_DATA, datalocal_dir))
        # Remove "marketplace" app and "gaiamobile.org" apps from webapps
        webapps_dir = datalocal_dir + self._LOCAL_DIR_DATA_APPS
        for root, dirs, files in os.walk(webapps_dir):
            if (os.path.basename(root).startswith('marketplace') or
                os.path.basename(root).endswith('gaiamobile.org') or
                os.path.basename(root).endswith('allizom.org')):
                logger.info('Removing Mozilla webapps: [{0}]'.format(root))
                shutil.rmtree(root)
        logger.info('Backup profile done.')

    def check_profile_version(self, local_dir, serial=None):
        logger.info('Checking profile...')
        # get local version
        if os.path.isdir(local_dir):
            local_config = ConfigParser.ConfigParser()
            local_config.read(local_dir + os.sep + self._LOCAL_DIR_B2G + os.sep + self._FILE_PROFILE_INI)
            local_profile_path = local_config.get('Profile0', 'Path')
            local_config.read(local_dir + os.sep + self._LOCAL_DIR_B2G + os.sep + local_profile_path + os.sep + self._FILE_COMPATIBILITY_INI)
            version_of_backup = local_config.get('Compatibility', 'LastVersion')
            logger.info('The Version of Backup Profile: {}'.format(version_of_backup))
        else:
            return False
        try:
            # get remote version
            tmp_dir = tempfile.mkdtemp(prefix='backup_restore_')
            try:
                AdbWrapper.adb_pull(self._REMOTE_DIR_B2G + os.sep + self._FILE_PROFILE_INI, tmp_dir, serial=serial)
            except:
                logger.warning('Can not pull {2} from {0} to {1}'.format(self._REMOTE_DIR_B2G, tmp_dir, self._FILE_PROFILE_INI))
                return False
            remote_config = ConfigParser.ConfigParser()
            remote_config.read(tmp_dir + os.sep + self._FILE_PROFILE_INI)
            remote_profile_path = remote_config.get('Profile0', 'Path')
            try:
                AdbWrapper.adb_pull(self._REMOTE_DIR_B2G + os.sep + remote_profile_path + os.sep + self._FILE_COMPATIBILITY_INI, tmp_dir, serial=serial)
            except:
                logger.warning('Can not pull {2} from {0} to {1}'.format(self._REMOTE_DIR_B2G, tmp_dir, self._FILE_COMPATIBILITY_INI))
                return False
            remote_config.read(tmp_dir + os.sep + self._FILE_COMPATIBILITY_INI)
            version_of_device = remote_config.get('Compatibility', 'LastVersion')
            logger.info('The Version of Device Profile: {}'.format(version_of_device))
            # compare
            version_of_backup_float = float(version_of_backup.split('.')[0])
            version_of_device_float = float(version_of_device.split('.')[0])
            if version_of_device_float >= version_of_backup_float:
                return True
            else:
                return False
        finally:
            logger.debug('Removing [{0}] folder...'.format(tmp_dir))
            shutil.rmtree(tmp_dir)

    def restore_profile(self, local_dir, serial=None):
        logger.info('Restoring profile...')
        if os.path.isdir(local_dir):
            # Restore Wifi
            wifi_file = local_dir + os.sep + self._LOCAL_FILE_WIFI
            if os.path.isfile(wifi_file):
                logger.info('Restoring Wifi information...')
                try:
                    AdbWrapper.adb_push(wifi_file, self._REMOTE_FILE_WIFI, serial=serial)
                except:
                    logger.warning('If you don\'t have root permission, you cannot restore Wifi information.')
                AdbWrapper.adb_shell('chown {0} {1}'.format(self._REMOTE_FILE_WIFI_OWNER, self._REMOTE_FILE_WIFI))
            # Restore profile
            b2g_mozilla_dir = local_dir + os.sep + self._LOCAL_DIR_B2G
            if os.path.isdir(b2g_mozilla_dir):
                logger.info('Restore from {0} to {1} ...'.format(b2g_mozilla_dir, self._REMOTE_DIR_B2G))
                AdbWrapper.adb_shell('rm -r {0}'.format(self._REMOTE_DIR_B2G))
                try:
                    AdbWrapper.adb_push(b2g_mozilla_dir, self._REMOTE_DIR_B2G, serial=serial)
                except:
                    logger.warning('Can not push files from {0} to {1}'.format(b2g_mozilla_dir, self._REMOTE_DIR_B2G))
            # Restore data/local
            datalocal_dir = local_dir + os.sep + self._LOCAL_DIR_DATA
            if os.path.isdir(datalocal_dir):
                logger.info('Restore from {0} to {1} ...'.format(datalocal_dir, self._REMOTE_DIR_DATA))
                AdbWrapper.adb_shell('rm -r {0}'.format(self._REMOTE_DIR_DATA))
                try:
                    AdbWrapper.adb_push(datalocal_dir, self._REMOTE_DIR_DATA, serial=serial)
                except:
                    logger.warning('Can not push files from {0} to {1}'.format(datalocal_dir, self._REMOTE_DIR_DATA))
            logger.info('Restore profile done.')
        else:
            logger.info('{0}: No such file or directory'.format(local_dir))
            return

    def run(self):
        # get the device's serial number
        devices = AdbWrapper.adb_devices()
        if len(devices) == 0:
            raise Exception('No device.')
        else:
            device_serial = AdbHelper.get_serial(self.args.serial)
            if device_serial is None:
                if len(devices) == 1:
                    logger.debug('No serial, and only one device')
                else:
                    logger.debug('No serial, but there are more than one device')
                    raise Exception('Please specify the device by --serial option.')
            else:
                logger.debug('Setup serial to [{0}]'.format(device_serial))

        # checking the adb root for backup/restore
        if not AdbWrapper.adb_root(serial=device_serial):
            raise Exception('No root permission for backup and resotre.')

        # Backup
        if self.args.backup:
            try:
                logger.info('Target device [{0}]'.format(device_serial))
                # Create temp folder
                tmp_dir = tempfile.mkdtemp(prefix='backup_restore_')
                # Stop B2G
                self.stop_b2g(serial=device_serial)
                # Backup User Profile
                self.backup_profile(local_dir=tmp_dir, serial=device_serial)
                # Backup SDCard
                if self.args.sdcard:
                    self.backup_sdcard(local_dir=tmp_dir, serial=device_serial)
                # Copy backup files from temp folder to target folder
                if os.path.isdir(self.args.profile_dir):
                    logger.warning('Removing [{0}] folder...'.format(self.args.profile_dir))
                    shutil.rmtree(self.args.profile_dir)
                logger.info('Copy profile from [{0}] to [{1}].'.format(tmp_dir, self.args.profile_dir))
                shutil.copytree(tmp_dir, self.args.profile_dir)
                # Start B2G
                if not self.args.no_reboot:
                    self.start_b2g(serial=device_serial)
            finally:
                logger.debug('Removing [{0}] folder...'.format(tmp_dir))
                shutil.rmtree(tmp_dir)
        # Restore
        elif self.args.restore:
            logger.info('Target device [{0}]'.format(device_serial))
            # Checking the Version of Profile
            if self.check_profile_version(local_dir=self.args.profile_dir, serial=device_serial):
                # Stop B2G
                self.stop_b2g(serial=device_serial)
                # Restore User Profile
                self.restore_profile(local_dir=self.args.profile_dir, serial=device_serial)
                # Restore SDCard
                if self.args.sdcard:
                    self.restore_sdcard(local_dir=self.args.profile_dir, serial=device_serial)
                # Start B2G
                if not self.args.no_reboot:
                    self.start_b2g(serial=device_serial)
            else:
                logger.warning('The version on device is smaller than backup\'s version.')


def main():
    try:
        BackupRestoreHelper().run()
    except Exception as e:
        logger.error(e)
        exit(1)


if __name__ == '__main__':
    main()



#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import os
import re
import logging
import argparse
from argparse import ArgumentDefaultsHelpFormatter
from util.adb_helper import AdbHelper
from util.adb_helper import AdbWrapper


logger = logging.getLogger(__name__)


class CrashReporter(object):
    def __init__(self, **kwargs):
        self.arg_parser = argparse.ArgumentParser(description='Get the Crash Reports from Firefox OS Phone.',
                                                  formatter_class=ArgumentDefaultsHelpFormatter)
        self.arg_parser.add_argument('-s', '--serial', action='store', dest='serial', default=None, help='Directs command to the device or emulator with the given serial number. Overrides ANDROID_SERIAL environment variable.')
        self.arg_parser.add_argument('-v', '--verbose', action='store_true', dest='verbose', default=False, help='Turn on verbose output, with all the debug logger.')
        self.args = self.arg_parser.parse_args()
        # setup the logging config
        if self.args.verbose is True:
            verbose_formatter = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
            logging.basicConfig(level=logging.DEBUG, format=verbose_formatter)
        else:
            formatter = '%(levelname)s: %(message)s'
            logging.basicConfig(level=logging.INFO, format=formatter)
        AdbWrapper.check_adb()

    def get_crashreports(self, serial=None):
        AdbWrapper.adb_root(serial=serial)
        logger.info('Getting Crash Reports...')

        pending, retcode_pending = AdbWrapper.adb_shell('ls -al /data/b2g/mozilla/Crash\ Reports/pending', serial=serial)
        print('Pending Crash Reports:\n{}\n'.format(pending))

        submitted, retcode_submitted = AdbWrapper.adb_shell('ls -al /data/b2g/mozilla/Crash\ Reports/submitted', serial=serial)
        print('Submitted Crash Reports:\n{}\n'.format(submitted))

        if retcode_submitted == 0:
            print('The links of Submitted Crash Reports:')
            for line in submitted.split('\n'):
                submmited_id = re.sub(r'\.txt\s*$', '', re.sub(r'^.+bp-', '', line))
                submitted_url = 'https://crash-stats.mozilla.com/report/index/{}'.format(submmited_id)
                print(submitted_url)

    def run(self):
        devices = AdbWrapper.adb_devices()

        if len(devices) == 0:
            raise Exception('No device.')
        elif len(devices) >= 1:
            final_serial = AdbHelper.get_serial(self.args.serial)
            if final_serial is None:
                if len(devices) == 1:
                    logger.debug('No serial, and only one device')
                    self.get_crashreports(serial=final_serial)
                else:
                    logger.debug('No serial, but there are more than one device')
                    raise Exception('Please specify the device by --serial option.')
            else:
                print('Serial: {0} (State: {1})'.format(final_serial, devices[final_serial]))
                self.get_crashreports(serial=final_serial)


def main():
    try:
        CrashReporter().run()
    except Exception as e:
        logger.error(e)
        exit(1)


if __name__ == '__main__':
    main()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import types


class Logger(object):

    _SYMBOL_CHAR = '###'
    _LEVEL_INFO = 'INFO'
    _LEVEL_WARNING = 'WARNING'
    _LEVEL_DEBUG = 'DEBUG'

    def log(self, message, status_callback=None, level='INFO'):
        print self._SYMBOL_CHAR + ' ' + level + ': ' + message
        if isinstance(status_callback, types.FunctionType) or isinstance(status_callback, types.MethodType):
            status_callback(message)



#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import re
import os
import json
import shutil
import logging
import zipfile
import tempfile
import argparse
import subprocess
from distutils import util
from datetime import datetime
from argparse import ArgumentDefaultsHelpFormatter
from util import console_utilities
from util.adb_helper import AdbWrapper
from util.adb_helper import AdbHelper
from util.adb_helper import AdbWrapper


logger = logging.getLogger(__name__)


class VersionChecker(object):
    def __init__(self, **kwargs):
        self.arg_parser = argparse.ArgumentParser(description='Check the version information of Firefox OS.',
                                                  formatter_class=ArgumentDefaultsHelpFormatter)
        self.arg_parser.add_argument('--no-color', action='store_true', dest='no_color', default=False, help='Do not print with color. NO_COLOR will overrides this option.')
        self.arg_parser.add_argument('-s', '--serial', action='store', dest='serial', default=None, help='Directs command to the device or emulator with the given serial number. Overrides ANDROID_SERIAL environment variable.')
        self.arg_parser.add_argument('--log-text', action='store', dest='log_text', default=None, help='Text ouput.')
        self.arg_parser.add_argument('--log-json', action='store', dest='log_json', default=None, help='JSON output.')
        self.arg_parser.add_argument('-v', '--verbose', action='store_true', dest='verbose', default=False, help='Turn on verbose output, with all the debug logger.')
        self.args = self.arg_parser.parse_args()
        # setup the logging config
        if self.args.verbose is True:
            verbose_formatter = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
            logging.basicConfig(level=logging.DEBUG, format=verbose_formatter)
        else:
            formatter = '%(levelname)s: %(message)s'
            logging.basicConfig(level=logging.INFO, format=formatter)
        AdbWrapper.check_adb()

    def get_device_info(self, serial=None):
        try:
            tmp_dir = tempfile.mkdtemp(prefix='checkversions_')
            # pull data from device
            try:
                AdbWrapper.adb_pull('/system/b2g/omni.ja', tmp_dir, serial=serial)
            except:
                logger.error('Error pulling Gecko file.')
            try:
                AdbWrapper.adb_pull('/data/local/webapps/settings.gaiamobile.org/application.zip', tmp_dir, serial=serial)
            except:
                try:
                    AdbWrapper.adb_pull('/system/b2g/webapps/settings.gaiamobile.org/application.zip', tmp_dir, serial=serial)
                except:
                    logger.error('Error pulling Gaia file.')
            try:
                AdbWrapper.adb_pull('/system/b2g/application.ini', tmp_dir, serial=serial)
            except:
                logger.error('Error pulling application.ini file.')
            # get Gaia info
            gaia_rev = 'n/a'
            gaia_date = 'n/a'
            application_zip_file = tmp_dir + os.sep + 'application.zip'
            if os.path.isfile(application_zip_file):
                f = open(application_zip_file, 'rb')
                z = zipfile.ZipFile(f)
                z.extract('resources/gaia_commit.txt', tmp_dir)
                f.close()
            else:
                logger.warning('Can not find application.zip file.')
            gaiacommit_file = tmp_dir + os.sep + 'resources/gaia_commit.txt'
            if os.path.isfile(gaiacommit_file):
                f = open(gaiacommit_file, "r")
                gaia_rev = re.sub(r'\n+', '', f.readline())
                gaia_date_sec_from_epoch = re.sub(r'\n+', '', f.readline())
                f.close()
                gaia_date = datetime.utcfromtimestamp(int(gaia_date_sec_from_epoch)).strftime('%Y-%m-%d %H:%M:%S')
            else:
                logger.warning('Can not get gaia_commit.txt file from application.zip file.')
            # deoptimize omni.ja for Gecko info
            gecko_rev = 'n/a'
            if os.path.isfile(tmp_dir + os.sep + 'omni.ja'):
                deopt_dir = tmp_dir + os.sep + 'deopt'
                deopt_file = deopt_dir + os.sep + 'omni.ja'
                deopt_exec = tmp_dir + os.sep + 'optimizejars.py'
                os.makedirs(deopt_dir)
                # TODO rewrite optimizejars.py if possible
                current_dir = cur = os.path.dirname(os.path.abspath(__file__))
                current_exec = os.path.join(current_dir, 'misc', 'optimizejars.py')
                shutil.copyfile(current_exec, deopt_exec)
                cmd = 'python %s --deoptimize %s %s %s' % (deopt_exec, tmp_dir, tmp_dir, deopt_dir)
                p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
                output = p.communicate()[0]
                # unzip omni.ja to get Gecko info
                if os.path.isfile(deopt_file):
                    f = open(deopt_file, 'rb')
                    z = zipfile.ZipFile(f)
                    z.extract('chrome/toolkit/content/global/buildconfig.html', tmp_dir)
                    f.close()
                else:
                    logger.warning('Can not deoptimize omni.ja file.')
                    gecko_rev = 'n/a'
                # get Gecko info from buildconfig.html file
                buildconfig_file = tmp_dir + os.sep + 'chrome/toolkit/content/global/buildconfig.html'
                if os.path.isfile(buildconfig_file):
                    for line in open(buildconfig_file, "r"):
                        if re.search(r'Built from', line):
                            ret = re.findall(r'>(.*?)<', line)
                            gecko_rev = ret[1]
                            break
                else:
                    logger.warning('Can not get buildconfig.html file from omni.ja file.')
            else:
                print 'Can not find omni.ja file.'
            # get Gecko version, and B2G BuildID from application.ini file
            if os.path.isfile(tmp_dir + os.sep + 'application.ini'):
                for line in open(tmp_dir + os.sep + 'application.ini', "r"):
                    if re.search(r'^\s*BuildID', line):
                        ret = re.findall(r'.*?=(.*)', line)
                        build_id = ret[0]
                    if re.search(r'^\s*Version', line):
                        ret = re.findall(r'.*?=(.*)', line)
                        version = ret[0]
            else:
                build_id = 'n/a'
                version = 'n/a'
            # get device information by getprop command
            device_name = re.sub(r'\r+|\n+', '', AdbWrapper.adb_shell('getprop ro.product.device', serial=serial)[0])
            firmware_release = re.sub(r'\r+|\n+', '', AdbWrapper.adb_shell('getprop ro.build.version.release', serial=serial)[0])
            firmware_incremental = re.sub(r'\r+|\n+', '', AdbWrapper.adb_shell('getprop ro.build.version.incremental', serial=serial)[0])
            firmware_date = re.sub(r'\r+|\n+', '', AdbWrapper.adb_shell('getprop ro.build.date', serial=serial)[0])
            firmware_bootloader = re.sub(r'\r+|\n+', '', AdbWrapper.adb_shell('getprop ro.boot.bootloader', serial=serial)[0])
            # prepare the return information
            device_info = {}
            device_info['Serial'] = serial
            device_info['Build ID'] = build_id
            device_info['Gaia Revision'] = gaia_rev
            device_info['Gaia Date'] = gaia_date
            device_info['Gecko Revision'] = gecko_rev
            device_info['Gecko Version'] = version
            device_info['Device Name'] = device_name
            device_info['Firmware(Release)'] = firmware_release
            device_info['Firmware(Incremental)'] = firmware_incremental
            device_info['Firmware Date'] = firmware_date
            device_info['Bootloader'] = firmware_bootloader
        finally:
            shutil.rmtree(tmp_dir)
        return device_info

    def _print_device_info_item(self, title, value, title_color=None, value_color=None):
        console_utilities.print_color('{0:22s}'.format(title), fg_color=title_color, newline=False)
        console_utilities.print_color(value, fg_color=value_color)

    def print_device_info(self, device_info, no_color=False):
        # setup the format by platform
        if no_color:
            title_color = None
            sw_color = None
            hw_color = None
        else:
            title_color = console_utilities.COLOR_LIGHT_BLUE
            sw_color = console_utilities.COLOR_LIGHT_GREEN
            hw_color = console_utilities.COLOR_LIGHT_YELLOW
        # print the device information
        self._print_device_info_item('Build ID', device_info['Build ID'], title_color=title_color, value_color=sw_color)
        self._print_device_info_item('Gaia Revision', device_info['Gaia Revision'], title_color=title_color, value_color=sw_color)
        self._print_device_info_item('Gaia Date', device_info['Gaia Date'], title_color=title_color, value_color=sw_color)
        self._print_device_info_item('Gecko Revision', device_info['Gecko Revision'], title_color=title_color, value_color=sw_color)
        self._print_device_info_item('Gecko Version', device_info['Gecko Version'], title_color=title_color, value_color=sw_color)
        self._print_device_info_item('Device Name', device_info['Device Name'], title_color=title_color, value_color=hw_color)
        self._print_device_info_item('Firmware(Release)', device_info['Firmware(Release)'], title_color=title_color, value_color=hw_color)
        self._print_device_info_item('Firmware(Incremental)', device_info['Firmware(Incremental)'], title_color=title_color, value_color=hw_color)
        self._print_device_info_item('Firmware Date', device_info['Firmware Date'], title_color=title_color, value_color=hw_color)
        if device_info['Bootloader'] is not '':
            self._print_device_info_item('Bootloader', device_info['Bootloader'], title_color=title_color, value_color=hw_color)
        print ''

    def output_log(self, device_info_list):
        if self.args.log_json is None and self.args.log_text is None:
            return
        # prepare the result dict for parsing
        result = {}
        unknown_serial_index = 1
        for device_info in device_info_list:
            if device_info['Serial'] == None:
                device_serial = 'unknown_serial_' + str(unknown_serial_index)
                unknown_serial_index = unknown_serial_index + 1
            else:
                device_serial = device_info['Serial']
            result[device_serial] = device_info
        # output
        if self.args.log_text is not None:
            with open(self.args.log_text, 'w') as outfile:
                for device_serial, device_info in result.items():
                    outfile.write('# %s\n' % device_serial)
                    if 'Skip' in device_info and device_info['Skip'] is True:
                        outfile.write('%s=%s\n' % ('Skip', device_info['Skip']))
                    else:
                        for key, value in device_info.items():
                            outfile.write('%s=%s\n' % (re.sub(r'\s+|\(|\)', '', key), re.sub(r'\s+', '_', value)))
                        outfile.write('\n')
        if self.args.log_json is not None:
            with open(self.args.log_json, 'w') as outfile:
                json.dump(result, outfile, indent=4)

    def run(self):
        devices = AdbWrapper.adb_devices()
        is_no_color = self.args.no_color
        if 'NO_COLOR' in os.environ:
            try:
                is_no_color = bool(util.strtobool(os.environ['NO_COLOR'].lower()))
            except:
                logger.error('Invalid NO_COLOR value [{0}].'.format(os.environ['NO_COLOR']))

        if len(devices) == 0:
            raise Exception('No device.')
        elif len(devices) >= 1:
            final_serial = AdbHelper.get_serial(self.args.serial)
            if final_serial is None:
                device_info_list = []
                for device, state in devices.items():
                    print('Serial: {0} (State: {1})'.format(device, state))
                    if state == 'device':
                        device_info = self.get_device_info(serial=device)
                        self.print_device_info(device_info, no_color=is_no_color)
                        device_info_list.append(device_info)
                    else:
                        print('Skipped.\n')
                        device_info_list.append({'Serial': device, 'Skip': True})
                self.output_log(device_info_list)
            else:
                print('Serial: {0} (State: {1})'.format(final_serial, devices[final_serial]))
                device_info = self.get_device_info(serial=final_serial)
                self.print_device_info(device_info, no_color=is_no_color)
                self.output_log([device_info])


def main():
    try:
        VersionChecker().run()
    except Exception as e:
        logger.error(e)
        exit(1)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import os
from sys import platform as _platform
from Tkinter import Tk
from tkFileDialog import askopenfilename
from tkMessageBox import askquestion
from tkMessageBox import askyesno


def main():
    window = Tk()
    window.option_add('*Dialog.msg.wrapLength', '10i')
    window.withdraw()

    ret = askyesno('Flash Local', 'It will shallow flash gaia/gecko from local file system.', icon='info')
    if ret:
        pass
    else:
        quit()

    is_flash_gaia = False
    GAIA_FILEOPENOPTIONS = {'title': 'Select the Gaia package (gaia.zip)', 'defaultextension': '.zip', 'filetypes': [('Zip file', '*.zip'), ('All files', '*.*')]}
    gaia_filename = askopenfilename(**GAIA_FILEOPENOPTIONS)
    if gaia_filename:
        is_flash_gaia = True

    is_flash_gecko = False
    GECKO_FILEOPENOPTIONS = {'title': 'Select the Gecko package (b2g-xxx.android-arm.tar.gz)', 'defaultextension': '.tar.gz', 'filetypes': [('Gzip file', '*.tar.gz'), ('All files', '*.*')]}
    gecko_filename = askopenfilename(**GECKO_FILEOPENOPTIONS)
    if gecko_filename:
        is_flash_gecko = True

    if not is_flash_gaia and not is_flash_gecko:
        quit()

    comfirm_message = 'Are You Sure?\n'
    if is_flash_gaia:
        comfirm_message = comfirm_message + '\nGaia:\n%s\n' % gaia_filename
    if is_flash_gecko:
        comfirm_message = comfirm_message + '\nGecko:\n%s\n' % gecko_filename

    ret = askyesno('Comfirm', comfirm_message, icon='warning')
    if ret:
        do_flash(is_flash_gaia, gaia_filename, is_flash_gecko, gecko_filename)
        comfirm_message = "Flash next device. " + comfirm_message
        while True:
            ret = askyesno('Comfirm', comfirm_message, icon='warning')
            if ret:
                do_flash(is_flash_gaia, gaia_filename, is_flash_gecko, gecko_filename)
            else:
                quit()
    else:
        quit()


def quit():
    print "Bye"
    exit(0)


def do_flash(is_flash_gaia, gaia_filename, is_flash_gecko, gecko_filename, keep_profile=False):
    cmd = './shallow_flash.sh -y'
    sp = ''
    if _platform == 'darwin':
        sp = ' '
    if is_flash_gaia:
        cmd = cmd + ' -g' + sp + gaia_filename
    if is_flash_gecko:
        cmd = cmd + ' -G' + sp + gecko_filename
    if keep_profile:
        cmd = cmd + ' --keep_profile'
    print('run command: ' + cmd)
    os.system(cmd)

if __name__ == '__main__':
    main()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import unittest
from logger import Logger


class testPathParser(unittest.TestCase):

    def setUp(self):
        self.logger = Logger()

    def test_logger_callback(self):
        '''
        Test the status callback of logger.
        '''
        test_message = 'TEST MESSAGE'
        self.logger.log(test_message, status_callback=self.mock_callback)
        print 'cb result', self.callback_message
        self.assertEqual(test_message, self.callback_message)

    def mock_callback(self, message):
        self.callback_message = message

if __name__ == '__main__':
    unittest.main()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import os
import types
import zipfile
import tarfile
from logger import Logger


class Decompressor(object):

    def __init__(self):
        self.logger = Logger()

    def unzip(self, source_file, dest_folder, status_callback=None):
        try:
            self.logger.log('Unzip ' + source_file + ' to ' + dest_folder, status_callback=status_callback)
            zip_file = zipfile.ZipFile(source_file)
            zip_file.extractall(dest_folder)
            zip_file.close()
            self.logger.log('Unzip done', status_callback=status_callback)
        except Exception as e:
            self.logger.log('Unzip Error ' + source_file, status_callback=status_callback, level=Logger._LEVEL_WARNING)

    def untar(self, source_file, dest_folder, status_callback=None):
        try:
            self.logger.log('Untar' + source_file + 'to' + dest_folder, status_callback=status_callback)
            tar_file = tarfile.open(source_file)
            tar_file.extractall(dest_folder)
            tar_file.close()
            self.logger.log('Untar done', status_callback=status_callback)
        except Exception as e:
            self.logger.log('Untar Error ' + source_file, status_callback=status_callback, level=Logger._LEVEL_WARNING)

    def ensure_folder(self, folder):
        if not os.path.isdir(folder):
            os.makedirs(folder)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import os
import re
import logging
import subprocess
from distutils import spawn


logger = logging.getLogger(__name__)


class AdbWrapper(object):

    @classmethod
    def check_adb(cls):
        logger.debug('Checking ADB...')
        if spawn.find_executable('adb') == None:
            raise Exception('There is no "adb" in your environment PATH.')
        logger.debug('You have ADB.')
        return True

    @classmethod
    def adb_devices(cls):
        '''
        Return devices as dict {device_serial: device_status, ...}.
        Rasise Exception when return code isn't zero.
        '''
        cmd = 'adb devices'
        p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        output, stderr = p.communicate()
        logger.debug('CMD: {0}'.format(cmd))
        logger.debug('RET: {0}'.format(output))
        if stderr:
            logger.debug('ERR: {0}'.format(stderr))
        if p.returncode is not 0:
            raise Exception('{}'.format({'STDOUT': output, 'STDERR': stderr}))
        output = re.sub(r'\r+', '', output)
        output_list = re.split(r'\n+', output)
        # remove '', '* daemon not running. starting it now on port xxx *', and '* daemon started successfully *'
        output_list = [item for item in output_list if item and not item.startswith('* daemon')]
        filter = re.compile(r'(^List of devices attached\s*|^\s+$)')
        output_list = [i for i in output_list if not filter.search(i)]
        output_list = [(re.split(r'\t', item)) for item in output_list if True]
        devices = {}
        for device in output_list:
            devices[device[0]] = device[1]
        return devices

    @classmethod
    def adb_pull(cls, source, dest, serial=None):
        '''
        Return stdout of command.
        Rasise Exception when return code isn't zero.
        '''
        if serial is None:
            cmd = 'adb pull'
        else:
            cmd = 'adb -s %s pull' % (serial,)
        cmd = '%s %s %s' % (cmd, source, dest)
        p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        output, stderr = p.communicate()
        logger.debug('CMD: {0}'.format(cmd))
        logger.debug('RET: {0}'.format(output))
        if stderr:
            logger.debug('ERR: {0}'.format(stderr))
        if p.returncode is not 0:
            raise Exception('{}'.format({'STDOUT': output, 'STDERR': stderr}))
        return output

    @classmethod
    def adb_push(cls, source, dest, serial=None):
        '''
        Return stdout of command.
        Rasise Exception when return code isn't zero.
        '''
        if serial is None:
            cmd = 'adb push'
        else:
            cmd = 'adb -s %s push' % (serial,)
        cmd = '%s %s %s' % (cmd, source, dest)
        p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        output, stderr = p.communicate()
        logger.debug('CMD: {0}'.format(cmd))
        logger.debug('RET: {0}'.format(output))
        if stderr:
            logger.debug('ERR: {0}'.format(stderr))
        if p.returncode is not 0:
            raise Exception('{}'.format({'STDOUT': output, 'STDERR': stderr}))
        return output

    @classmethod
    def adb_shell(cls, command, serial=None):
        '''
        Return the stdout and return code (from device) of "adb shell" command.
        Raise exception when return code (from adb command) isn't zero.
        '''
        if serial is None:
            cmd = 'adb shell'
        else:
            cmd = 'adb -s %s shell' % (serial,)
        # get returncode from device
        cmd = "%s '%s; echo $?'" % (cmd, command)
        p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        shell_ret, stderr = p.communicate()
        logger.debug('CMD: {0}'.format(cmd))
        logger.debug('RAW_RET: {0}'.format(shell_ret))
        if stderr:
            logger.debug('RAW_ERR: {0}'.format(stderr))
        if p.returncode is not 0:
            raise Exception('{}'.format({'STDOUT': shell_ret, 'STDERR': stderr}))
        # split the stdout and retcode (from device)
        shell_ret = re.sub(r'\s+$', '', shell_ret)
        shell_output = re.split(r'\s+(\d+$)', shell_ret, maxsplit=1)
        # when no stdout, set output to ''
        if len(shell_output) == 1:
            # only returncode, no stdout
            output = ''
            try:
                returncode = int(shell_output[0])
            except:
                # some command will stop device with no returncode. e.g. adb shell reboot recovery
                returncode = 0
        else:
            # stdout with returncode
            output = shell_output[0]
            try:
                returncode = int(shell_output[1])
            except:
                returncode = 0
        logger.debug('RET: {0}'.format(output))
        logger.debug('RET CODE: {0}'.format(returncode))
        return output, returncode

    @classmethod
    def adb_root(cls, serial=None):
        '''
        Return True if adb already running as root.
        '''
        if serial is None:
            cmd = 'adb root'
        else:
            cmd = 'adb -s %s root' % (serial,)
        p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        output, stderr = p.communicate()
        logger.debug('cmd: {0}'.format(cmd))
        logger.debug('output: {0}'.format(output))
        if stderr:
            logger.debug('ERR: {0}'.format(stderr))
        if p.returncode is 0 and (not 'cannot' in output):
            if 'restarting' in output:
                import time
                time.sleep(1)
            logger.debug('adb root successed')
            logger.info('{}'.format(output))
            return True
        else:
            logger.debug('adb root failed')
            logger.warning('{}'.format(output))
            return False


class AdbHelper(object):

    @classmethod
    def get_serial(cls, serial_number):
        '''
        Input the serial number.
        This method will check with "ANDROID_SERIAL" environmental variable, then return the serial number if the serial is avaiable.
        When serial number is not avaiable, return None.
        ex:
            \ANDROID_SERIAL
        serial\ |  O  |  X
        --------+-----+------
              O |  s  |  s
        --------+-----+------
              X | A_S | None
        '''
        # return None if there are no serial and ANDROID_SERIAL
        final_serial_number = None
        # no serial then check ANDROID_SERIAL
        if serial_number is None or serial_number == '':
            if 'ANDROID_SERIAL' in os.environ:
                logger.debug('ANDROID_SERIAL={}'.format(os.environ.get('ANDROID_SERIAL')))
                final_serial_number = os.environ.get('ANDROID_SERIAL')
        else:
            final_serial_number = serial_number
            logger.debug('serial={}'.format(serial_number))
        # raise Exception if the serial is not in devices list
        if final_serial_number is not None:
            devices = AdbWrapper.adb_devices()
            logger.debug('Devices: {}'.format(devices))
            if not final_serial_number in devices:
                raise Exception('Can not found {} device in devices list {}.'.format(final_serial_number, devices))
        return final_serial_number
